FILE_TYPE = 'file_type'
FILE_MASK = 'file_mask'
DELIMITED = 'DELIMITED'
FIXED = 'FIXED'
XML = 'XML'
DELIMITER = 'delimiter'
HAS_HEADER = 'has_header'
FIELD_DEFINITIONS = 'field_definitions'
FIELD_NAME = 'field_name'
START_POSITION = 'start_position'
FIELD_LENGTH = 'field_length'
RECORD_ELEMENT_NAME = 'record_element_name'
XML_NODE_NAME = 'xml_node_name'
INPUT_DIRECTORY = 'input_directory'
COMPLETED_DIRECTORY = 'completed_directory'
POSITION_IN_ROW = 'position_in_row'
